from .externally import ExternallyTagged
from .adjacently import AdjacentlyTagged
from .internally import InternallyTagged

__all__ = [
    "ExternallyTagged",
    "AdjacentlyTagged",
    "InternallyTagged",
]
